@@ -15,7 +15,7 @@ from GraphLoader import GraphLoader
 
 def page_rank(G, name=None, max_iterations=10):
     '''Compute page rank in parallel for the networkx graph G.'''
-    os.system("echo "+str(1.0/float(len(G))) + " > rank_mass")
+    
     wd = config.GraphReduce_location
     ranks = dict(zip(map(str,G.nodes()),[1.0/len(G)]*len(G)))
     G = GraphLoader(G)
@@ -32,7 +32,9 @@ def page_rank(G, name=None, max_iterations=10):
 def parallel_page_rank(G, hdfs_handle, old_ranks, iterations, max_iterations):
     '''Compute page rank in parallel for the networkx graph G.'''
     hdfs.rm("PPR")
-    hadoop_call = hdmc.build_generic_hadoop_call("PageRank_mapper.py", "PageRank_reducer.py", hdfs_handle+"/page_rank", "PPR", ["rank_mass"])
+    base_path =  os.path.realpath( __file__ ).split("/")
+    base_path = "/".join(base_path[0:-1])
+    hadoop_call = hdmc.build_generic_hadoop_call(base_path+"/PageRank_mapper.py", base_path+"/PageRank_reducer.py", hdfs_handle+"/page_rank", "PPR", ["rank_mass"])
     hdmc.execute_and_wait(hadoop_call)
     listing = hdfs.ls("PPR/part*")["stdout"].rstrip().split("\n")
     for entry in listing:
@@ -48,7 +50,7 @@ def parallel_page_rank(G, hdfs_handle, old_ranks, iterations, max_iterations):
         hdfs.rm(hdfs_handle+"/page_rank/part*")
         hdfs.mv("PPR/part*", hdfs_handle+"/page_rank/")
         hdfs.rm("PPR")
-        hadoop_call = hdmc.build_generic_hadoop_call("LostMass_mapper.py", "LostMass_reducer.py", hdfs_handle+"/page_rank", "PPR", ["rank_mass", "lost_mass"])
+        hadoop_call = hdmc.build_generic_hadoop_call(base_path+"/LostMass_mapper.py", base_path+"/LostMass_reducer.py", hdfs_handle+"/page_rank", "PPR", ["rank_mass", "lost_mass"])
         hdmc.execute_and_wait(hadoop_call)
 
         
