@@ -23,15 +23,20 @@ import sys
 import os
 import stat
 from code_generator import CodeGenerator
+from glob import glob
 
 def make_checkpointing_filter():
     '''Generates a python script which filters checkpointing results from HDMC.'''
     c = CodeGenerator()
     c.begin()
     c.write("#! /usr/bin/env python\n\n")
+    c.write("from copy import copy\n")
     c.write("import sys\n\n")
+    c.write("current_key = ''\n")
     c.write("def read_input(file):\n")
     c.indent()
+    c.write("global current_key\n")
+    c.write("global current_line\n")
     c.write("for line in file:\n")
     c.indent()
     c.write("line = line.strip()\n")
@@ -41,27 +46,46 @@ def make_checkpointing_filter():
     c.dedent()
     c.write("except ValueError:\n")
     c.indent()
-    c.write("key='moredata'\n")
+    c.write("key=current_key+'-moredata'\n")
     c.write("value=line\n")
     c.dedent()
+    c.write("current_key = key\n")
     c.write("yield key, value\n")
     c.dedent()
     c.dedent()
     c.write("\n\n")
     c.write("def main():\n")
     c.indent()
-    c.write("seen_keys = []\n")
+    c.write("global current_key\n")
+    c.write("global current_line\n")
+    c.write("seen_keys = {}\n")
     c.write("data = read_input(sys.stdin)\n")
     c.write("for key, value in data:\n")
     c.indent()
-    c.write("if key=='moredata':\n")
+    c.write("line_data = value.split('==')\n")
+    c.write("line_key = int(line_data[1])\n")
+    c.write("line_value = line_data[2]\n")
+    c.write("if key in seen_keys:\n")
     c.indent()
-    c.write("print value\n")
+    c.write("if line_key not in seen_keys[key]:\n")
+    c.indent()
+    c.write("seen_keys[key][line_key] = line_value\n")
     c.dedent()
-    c.write("elif len(value)>0 and key not in seen_keys:\n")
+    c.dedent()
+    c.write("else:\n")
     c.indent()
-    c.write("seen_keys.append(key)\n")
-    c.write("print value\n")
+    c.write("seen_keys[key] = {}\n")
+    c.write("seen_keys[key][line_key] = line_value\n")
+    c.dedent()
+    c.dedent()
+    c.write("for key in seen_keys:\n")
+    c.indent()
+    c.write("lines = seen_keys[key].keys()\n")
+    c.write("lines.sort()\n")
+    c.write("for l in lines:\n")
+    c.indent()
+    #c.write("print key +':'+str(l)+':'+seen_keys[key][l]\n")
+    c.write("print seen_keys[key][l]\n")
     c.dedent()
     c.dedent()
     c.dedent()
@@ -76,10 +100,11 @@ def make_checkpointing_filter():
     frame_file.close()
     os.system("chmod a+rwx checkpoint_filter.py")
     
-def make_checkpointing_frame(script, checkpoint_names, checkpoint_dir,arguments="", debug=False):
+    
+def make_checkpointing_frame(script, checkpoint_names, checkpoint_dir,arguments="", files = True, debug=False):
     '''Generates a python script which given a list of files to be processed, 
 executes the specified script in over the files in parallel via MapReduce.'''
-    
+
     c = CodeGenerator()
     c.begin()
     c.write("#! /usr/bin/env python\n\n")
@@ -123,7 +148,17 @@ executes the specified script in over the files in parallel via MapReduce.'''
     c.write("subprocess.call(['chmod','777','"+checkpoint_dir+"'+'/'+this_checkpoint+'_start'])\n")
     cmd = str(shlex.split("./"+script.split("/")[-1] + " " + arguments))
     c.write("os.system('chmod a+rwx "+script.split("/")[-1]+"')\n")
-    c.write("cmd = "+cmd+"+['./'+this_checkpoint]\n")
+    if files:
+        c.write("cmd = "+cmd+"+['./'+this_checkpoint]\n")
+    else:
+        c.write("argf = open('./'+this_checkpoint).readlines()\n")
+        c.write("for i in range(len(argf)):\n")
+        c.indent()
+        c.write("argf[i] = argf[i].strip()\n")
+        c.dedent()
+        
+        c.write("cmd = "+cmd+"+argf\n")
+        
     c.write("p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)\n")
     c.write("output, error = p.communicate()\n")
     c.write("sts = p.wait()\n")
@@ -131,10 +166,17 @@ executes the specified script in over the files in parallel via MapReduce.'''
         c.write("output = output.strip()\n")
         c.write("if len(output) > 0:\n")
         c.indent()
-        c.write("print this_checkpoint+'==HDMC_CHECKPOINT=='+ output\n")
+        c.write("line_count = 0\n")
+        c.write("newline = '\\n'\n")
+        c.write("for line in output.split(newline):\n")
+        c.indent()
+        c.write("print this_checkpoint+'==HDMC_CHECKPOINT==LINE=='+ str(line_count) + '=='+line.strip()\n")
+        c.write("line_count += 1\n")
+        c.dedent()
         c.dedent()
         c.write("if len(error.strip()) > 0:\n")
         c.indent()
+        c.write("print >> sys.stderr, error.strip()\n")
         c.write("os.system('rm "+checkpoint_dir+"'+'/'+this_checkpoint)\n")
         c.write("exit(1)\n")
         c.dedent()
@@ -146,6 +188,8 @@ executes the specified script in over the files in parallel via MapReduce.'''
         c.write("os.system('rm "+checkpoint_dir+"/'+this_checkpoint+'_start')\n")
     else:
         c.write("print output.strip(),error.strip()\n")
+    c.write("subprocess.call(['touch','"+checkpoint_dir+"'+'/'+this_checkpoint+'_end'])\n")
+
     c.write("this_checkpoint = random.choice(all_checkpoints)\n")
     c.write("current_checkpoints = glob('"+checkpoint_dir+"/*')\n")
     c.dedent()
@@ -160,6 +204,8 @@ executes the specified script in over the files in parallel via MapReduce.'''
     print >> frame_file, c.end()
     frame_file.close()
     os.system("chmod a+rwx checkpoint_frame.py")
+    if not files:
+        os.system("cp checkpoint_frame.py checkpoint_frame_save.py")
     
 def make_frame(script, arguments="", iterations=1, debug=False):
     '''Generates a basic python frame for running a batch job on a MapReduce cluster.'''
@@ -199,12 +245,18 @@ def get_output_hdfs_name(output_data_name):
     return output_path[len(output_path)-1]
 
 
-def build_hadoop_call(script, output_data_name, iterations=1, supporting_file_list = None, reduction_script=None, arguments=None, debug = False ):
+def build_hadoop_call(script, output_data_name, iterations=1, supporting_file_list = None, reduction_script=None, arguments=None, debug=False, num_mappers=None, num_reducers=None):
     '''Builds a call array suitable for subprocess.Popen which submits a streaming job to 
 the configured MapReduce instance.  The function also generates the necessary execution frame.'''
     # I/O setup
-    hadoop_call = [config.hadoop, 'jar', config.hadoop_streaming,\
-                   '-input', 'dummy', '-output', get_output_hdfs_name(output_data_name)]
+    hadoop_call = [config.hadoop, 'jar', config.hadoop_streaming]
+    if num_mappers:    
+        hadoop_call += ["-D", "mapred.map.tasks="+str(num_mappers)]
+    if num_reducers:
+        hadoop_call += ["-D", "mapred.reduce.tasks="+str(num_reducers)]
+        
+    
+    hadoop_call += ['-input', 'dummy', '-output', get_output_hdfs_name(output_data_name)]
     #mapper name
     hadoop_call += ['-mapper', "frame.py"]
     
@@ -227,11 +279,18 @@ the configured MapReduce instance.  The function also generates the necessary ex
     make_frame(script, arguments, iterations, debug)
     return hadoop_call
 
-def build_checkpoint_call(script, output_data_name, supporting_file_list, reduction_script=None, arguments=None ):
+def build_checkpoint_call(script, output_data_name, supporting_file_list, reduction_script=None, arguments=None, num_mappers=None, num_reducers=None):
     '''Builds a call array suitable for subprocess.Popen which submits a streaming job to 
      the configured MapReduce instance.  The function also generates the necessary execution frame.'''
     # I/O setup
-    hadoop_call = [config.hadoop, 'jar', config.hadoop_streaming,'-input', 'dummy', '-output', get_output_hdfs_name(output_data_name)]
+    hadoop_call = [config.hadoop, 'jar', config.hadoop_streaming]
+    if num_mappers:    
+        hadoop_call += ["-D", "mapred.map.tasks="+str(num_mappers)]
+        
+    if num_reducers:
+        hadoop_call += ["-D", "mapred.reduce.tasks="+str(num_reducers)]
+        
+    hadoop_call += ['-input', 'dummy', '-output', get_output_hdfs_name(output_data_name)]
     #mapper name
     hadoop_call += ['-mapper', "checkpoint_frame.py"]
     
@@ -245,7 +304,8 @@ def build_checkpoint_call(script, output_data_name, supporting_file_list, reduct
     file_list = ["-file", script]
     file_list += ["-file", "./checkpoint_frame.py"]
     if reduction_script:
-        file_list += ["-file", reduction_script]
+        if reduction_script != "NONE":
+            file_list += ["-file", reduction_script]
     if supporting_file_list:
         for f in supporting_file_list:
             file_list += ["-file", f]
@@ -340,7 +400,8 @@ is always named hdmc_checkpoints and exists at the same level as the trailing en
 in output_data_name.'''    
     output_path = output_data_name.split("/")
     output_path.pop()
-    output_dir = config.shared_tmp_space+os.getlogin()
+    output_dir = config.shared_tmp_space+"/"+os.getlogin()
+    print output_dir
     try:
         os.mkdir(output_dir)
         os.system('chmod 777 '+ output_dir)
@@ -359,34 +420,51 @@ def get_checkpoint_names(file_list):
     the last member of the array generated by splitting in /'''
     checkpoints = []
     for f in file_list:
-        path = f.split("/")
-        checkpoints.append(path[-1])
+        if "/" in f:
+            path = f.split("/")
+            checkpoints.append(path[-1])
+        else:
+            path=f
+            checkpoints.append(f)
     return checkpoints
 
+def make_pseudo_checkpoints(file_list):
+    '''Designed to make checkpointing long lists of parameters (e.g. URLs) easier'''
+    checkpoints = []
+    os.system('mkdir pseudo_checkpoints')
+    cp_count = 0
+    for f in file_list:
+        os.system('echo '+f + "> pseudo_checkpoints/" + str(cp_count))
+        checkpoints.append(str(cp_count))
+        file_list[cp_count] = "pseudo_checkpoints/" + str(cp_count)
+        cp_count += 1
+    return checkpoints
+    
 
-def submit(script, output_data_name, iterations=1, supporting_file_list = None, reduction_script=None, arguments="", debug=False):
+
+def submit(script, output_data_name, iterations=1, supporting_file_list = None, reduction_script=None, arguments="", debug=False, num_mappers=None, num_reducers=None):
     '''Submits script non-blocking job to a MapReduce cluster and collects output
 in output_data_name.  Supporting filenames can be passed
 as a list, as can a reducing/filtering script.  Arguments to the submitted script
 should be passed as a string.'''    
     create_dummy_data()
-    hadoop_call = build_hadoop_call(script, output_data_name, iterations, supporting_file_list, reduction_script, arguments, debug)
+    hadoop_call = build_hadoop_call(script, output_data_name, iterations, supporting_file_list, reduction_script, arguments, debug, num_mappers, num_reducers)
     execute(hadoop_call)
     
 
 
-def submit_inline(script, output_data_name, iterations=1, supporting_file_list = None, reduction_script=None, arguments="", debug=False):
+def submit_inline(script, output_data_name, iterations=1, supporting_file_list = None, reduction_script=None, arguments="", debug=False, num_mappers=None, num_reducers=None):
     '''Submits script blocking job to a MapReduce cluster and collects output
 in output_data_name.  Supporting filenames can be passed
 as a list, as can a reducing/filtering script.  Arguments to the submitted script
 should be passed as a string.'''      
     create_dummy_data()
-    hadoop_call = build_hadoop_call(script, output_data_name, iterations, supporting_file_list, reduction_script, arguments, debug)
+    hadoop_call = build_hadoop_call(script, output_data_name, iterations, supporting_file_list, reduction_script, arguments, debug, num_mappers, num_reducers)
     execute_and_wait(hadoop_call)
     download_hdfs_data(output_data_name)
     cleanup()
     
-def submit_checkpoint_inline(script, output_data_name, file_list, reduction_script = None, arguments="", debug=False):
+def submit_checkpoint_inline(script, output_data_name, file_list, supporting_file_list=[], reduction_script = None, arguments="", files=True, debug=False, num_mappers=None, num_reducers=None):
     '''Submits a script to a MapReduce cluster for 
     parallel operation on a number of files.  An optional reducer script can be 
     applied as well, but should filter the map results by splitting file output
@@ -394,18 +472,28 @@ def submit_checkpoint_inline(script, output_data_name, file_list, reduction_scri
     should be passed as a string. Blocking.'''  
     create_dummy_data()
     checkpoint_dir = set_checkpoint_directory(output_data_name)
-    checkpoints = get_checkpoint_names(file_list)
-    make_checkpointing_frame(script, checkpoints, checkpoint_dir, arguments, debug)
+    if files:
+        checkpoints = get_checkpoint_names(file_list)
+    else:
+        print "not processing files"
+        checkpoints = make_pseudo_checkpoints(file_list)
+        
+    make_checkpointing_frame(script, checkpoints, checkpoint_dir, arguments, files, debug)
     if not reduction_script:
-        reduction_script = ("checkpoint_filter.py")
+        reduction_script = "checkpoint_filter.py"
         make_checkpointing_filter()
-    hadoop_call = build_checkpoint_call(script, output_data_name, file_list, reduction_script, arguments)
+#    elif not files:
+#        reduction_script = "NONE"
+        
+    hadoop_call = build_checkpoint_call(script, output_data_name, file_list+supporting_file_list, reduction_script, arguments, num_mappers, num_reducers)
+    
+    print "executing"
     execute_and_wait(hadoop_call)
     download_hdfs_data(output_data_name)
     cleanup()
     return checkpoints
 
-def submit_checkpoint(script, output_data_name, file_list, reduction_script = None, arguments="", debug=False):
+def submit_checkpoint(script, output_data_name, file_list, supporting_file_list=[], reduction_script = None, arguments="", files=True, debug=False, num_mappers=None, num_reducers=None):
     '''Submits a script to a MapReduce cluster for 
     parallel operation on a number of files.  An optional reducer script can be 
     applied as well, but should filter the map results by splitting file output
@@ -413,13 +501,25 @@ def submit_checkpoint(script, output_data_name, file_list, reduction_script = No
     should be passed as a string.  Non-blocking.'''  
     create_dummy_data()
     checkpoint_dir = set_checkpoint_directory(output_data_name)
-    checkpoints = get_checkpoint_names(file_list)
-    make_checkpointing_frame(script, checkpoints, checkpoint_dir, arguments, debug)
+    if files:
+        checkpoints = get_checkpoint_names(file_list)
+    else:
+        print "not processing files"
+        checkpoints = make_pseudo_checkpoints(file_list)
+        
+    make_checkpointing_frame(script, checkpoints, checkpoint_dir, arguments, files, debug)
     if not reduction_script:
-        reduction_script = ("checkpoint_filter.py")
+        reduction_script = "checkpoint_filter.py"
         make_checkpointing_filter()
-    hadoop_call = build_checkpoint_call(script, output_data_name, file_list, reduction_script, arguments)
-    execute(hadoop_call)
+#    elif not files:
+#        reduction_script = "NONE"
+        
+    hadoop_call = build_checkpoint_call(script, output_data_name, file_list+supporting_file_list, reduction_script, arguments, num_mappers, num_reducers)
+    
+    print "executing"
+    execute_and_wait(hadoop_call)
+    download_hdfs_data(output_data_name)
+    cleanup()
     return checkpoints
     
 def cleanup():
@@ -432,5 +532,10 @@ def cleanup():
         os.remove("checkpoint_filter.py")
     if os.path.isfile("dummy"):
         os.remove("dummy")
+    if os.path.isdir("pseudo_checkpoints"):
+        pcps = glob("pseudo_checkpoints/*")
+        for f in pcps:
+            os.remove(f)
+        os.removedirs("pseudo_checkpoints")
     
     
