@@ -8,7 +8,8 @@ def read_entries(file):
         key, value = line.split('==HDMC_CHECKPOINT==')
         value = value.strip()
         if len(value) > 0:
-            yield key, int(value.split()[0])
+            line_data = value.split("==")[2]
+            yield key, int(line_data.split()[0])
         
 def main():
     linecounts = read_entries(sys.stdin)
