@@ -82,6 +82,43 @@ def make_counting_frame():
     frame_file.close()
     os.system("chmod a+rwx histogram_frame.py")
     
+def make_grep_frame():
+    c = CodeGenerator()
+    c.begin()
+    c.write("#! /usr/bin/env python\n\n")
+    c.write("import sys\n\n")
+    c.write("def read_input(file, pattern):\n")
+    c.indent()
+    c.write("line_count = 0\n")
+    c.write("for line in file:\n")
+    c.indent()
+    c.write("line_count += 1\n")
+    c.write("if pattern in line.strip():\n")
+    c.indent()
+    c.write("yield line.strip()\n")
+    c.dedent()
+    c.dedent()
+    c.dedent()
+    c.write("\n\n")
+    c.write("def main():\n")
+    c.indent()
+    c.write("search_pattern = open('ziggy_search').readline().strip()\n")
+    c.write("data = read_input(sys.stdin, search_pattern)\n")
+    c.write("for line in data:\n")
+    c.indent()
+    c.write("print line\n")
+    c.dedent()
+    c.dedent()
+    c.write("\n\n")
+    c.write('if __name__ == \"__main__\":\n')
+    c.indent()
+    c.write("main()\n")
+    c.write("\n")
+    frame_file = open("search_frame.py", "w")
+    print >> frame_file, c.end()
+    frame_file.close()
+    os.system("chmod a+rwx search_frame.py")
+    
 def make_search_frame(suppress_lines):
     c = CodeGenerator()
     c.begin()
@@ -185,6 +222,26 @@ def sort_ascii(input_file, output_file, ascending=True,num_mappers=None, num_red
     hdfs.copyFromHDFS(output_file, output_file)
     cleanup()
     
+def text_search(input_directory, input_files, output_file, search_pattern, suppress_lines=True, num_mappers=None, num_reducers=None):
+    '''Use MapReduce to search a collection of input files.'''
+    #make an HDFS directory for the input files
+    hdfs.mkdir(input_directory)
+    for f in input_files:
+        hdfs_location = input_directory+"/"+f.split("/")[-1]
+        hdfs.copyToHDFS(f, hdfs_location)
+        
+     #make a special input for the search pattern
+    os.system("echo " + search_pattern + " > ziggy_search")
+    #make a search frame
+    make_grep_frame()
+    #make an identity frame
+    make_identity_frame()
+    #build the hadoop call
+    hadoop_call = hdmc.build_generic_hadoop_call("search_frame.py", "identity_frame.py", input_directory, output_file, ["./ziggy_search"], num_mappers, num_reducers)
+    hdmc.execute_and_wait(hadoop_call)
+    hdfs.copyFromHDFS(output_file, output_file)
+    cleanup()
+    
 def search(input_directory, input_files, output_file, search_pattern, suppress_lines=True, num_mappers=None, num_reducers=None):
     '''Use MapReduce to search a collection of input files.'''
     #make an HDFS directory for the input files
